@@ -7,10 +7,7 @@
 //!
 //! ## Examples
 //!
-//! With [`std::env::args()`](https://doc.rust-lang.org/std/env/fn.args.html):
-//!
 //! ```rust
-//! # // This should be kept in sync with the equivalent example in the readme.
 //! # extern crate cargo_metadata;
 //! # use std::path::Path;
 //! let mut args = std::env::args().skip_while(|val| !val.starts_with("--manifest-path"));
@@ -29,85 +26,6 @@
 //! let _metadata = cmd.exec().unwrap();
 //! ```
 //!
-//! With [`docopt`](https://docs.rs/docopt):
-//!
-//! ```rust
-//! # // This should be kept in sync with the equivalent example in the readme.
-//! # extern crate cargo_metadata;
-//! # extern crate docopt;
-//! # #[macro_use] extern crate serde_derive;
-//! # use std::path::Path;
-//! # use docopt::Docopt;
-//! # fn main() {
-//! const USAGE: &str = "
-//!     Cargo metadata test function
-//!
-//!     Usage:
-//!       cargo_metadata [--manifest-path PATH]
-//! ";
-//!
-//! #[derive(Debug, Deserialize)]
-//! struct Args {
-//!     arg_manifest_path: Option<String>,
-//! }
-//!
-//! let args: Args = Docopt::new(USAGE)
-//!     .and_then(|d| d.deserialize())
-//!     .unwrap_or_else(|e| e.exit());
-//!
-//! let mut cmd = cargo_metadata::MetadataCommand::new();
-//! if let Some(path) = args.arg_manifest_path {
-//!     cmd.manifest_path(path);
-//! }
-//! let _metadata = cmd.exec().unwrap();
-//! # }
-//! ```
-//!
-//! With [`clap`](https://docs.rs/clap):
-//!
-//! ```rust
-//! # // This should be kept in sync with the equivalent example in the readme.
-//! # extern crate cargo_metadata;
-//! # extern crate clap;
-//! let matches = clap::App::new("myapp")
-//!     .arg(
-//!         clap::Arg::with_name("manifest-path")
-//!             .long("manifest-path")
-//!             .value_name("PATH")
-//!             .takes_value(true),
-//!     )
-//!     .get_matches();
-//!
-//! let mut cmd = cargo_metadata::MetadataCommand::new();
-//! if let Some(path) = matches.value_of("manifest-path") {
-//!     cmd.manifest_path(path);
-//! }
-//! let _metadata = cmd.exec().unwrap();
-//! ```
-//! With [`structopt`](https://docs.rs/structopt):
-//!
-//! ```rust
-//! # // This should be kept in sync with the equivalent example in the readme.
-//! # extern crate cargo_metadata;
-//! # #[macro_use] extern crate structopt;
-//! # use std::path::PathBuf;
-//! # use structopt::StructOpt;
-//! # fn main() {
-//! #[derive(Debug, StructOpt)]
-//! struct Opt {
-//!     #[structopt(name = "PATH", long="manifest-path", parse(from_os_str))]
-//!     manifest_path: Option<PathBuf>,
-//! }
-//!
-//! let opt = Opt::from_args();
-//! let mut cmd = cargo_metadata::MetadataCommand::new();
-//! if let Some(path) = opt.manifest_path {
-//!     cmd.manifest_path(path);
-//! }
-//! let _metadata = cmd.exec().unwrap();
-//! # }
-//! ```
-//!
 //! Pass features flags
 //!
 //! ```rust
@@ -133,12 +51,13 @@
 //! use cargo_metadata::Message;
 //!
 //! let mut command = Command::new("cargo")
-//!     .args(&["build", "--message-format=json"])
+//!     .args(&["build", "--message-format=json-render-diagnostics"])
 //!     .stdout(Stdio::piped())
 //!     .spawn()
 //!     .unwrap();
 //!
-//! for message in cargo_metadata::parse_messages(command.stdout.take().unwrap()) {
+//! let reader = std::io::BufReader::new(command.stdout.take().unwrap());
+//! for message in cargo_metadata::Message::parse_stream(reader) {
 //!     match message.unwrap() {
 //!         Message::CompilerMessage(msg) => {
 //!             println!("{:?}", msg);
@@ -149,6 +68,9 @@
 //!         Message::BuildScriptExecuted(script) => {
 //!             println!("{:?}", script);
 //!         },
+//!         Message::BuildFinished(finished) => {
+//!             println!("{:?}", finished);
+//!         },
 //!         _ => () // Unknown message
 //!     }
 //! }
@@ -156,27 +78,28 @@
 //! let output = command.wait().expect("Couldn't get cargo's exit status");
 //! ```
 
-extern crate semver;
-extern crate serde;
-#[macro_use]
-extern crate serde_derive;
-extern crate serde_json;
-
+use camino::Utf8PathBuf;
+#[cfg(feature = "builder")]
+use derive_builder::Builder;
 use std::collections::HashMap;
 use std::env;
 use std::fmt;
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
 use std::process::Command;
 use std::str::from_utf8;
 
-use semver::Version;
+pub use camino;
+pub use semver::{Version, VersionReq};
 
 pub use dependency::{Dependency, DependencyKind};
 use diagnostic::Diagnostic;
 pub use errors::{Error, Result};
+#[allow(deprecated)]
+pub use messages::parse_messages;
 pub use messages::{
-    parse_messages, Artifact, ArtifactProfile, BuildScript, CompilerMessage, Message,
+    Artifact, ArtifactProfile, BuildFinished, BuildScript, CompilerMessage, Message, MessageIter,
 };
+use serde::{Deserialize, Serialize};
 
 mod dependency;
 pub mod diagnostic;
@@ -201,7 +124,18 @@ impl std::fmt::Display for PackageId {
     }
 }
 
+// Helpers for default metadata fields
+fn is_null(value: &serde_json::Value) -> bool {
+    match value {
+        serde_json::Value::Null => true,
+        _ => false,
+    }
+}
+
 #[derive(Clone, Serialize, Deserialize, Debug)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[non_exhaustive]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
 /// Starting point for metadata returned by `cargo metadata`
 pub struct Metadata {
     /// A list of all crates referenced by this crate (and the crate itself)
@@ -211,13 +145,22 @@ pub struct Metadata {
     /// Dependencies graph
     pub resolve: Option<Resolve>,
     /// Workspace root
-    pub workspace_root: PathBuf,
+    pub workspace_root: Utf8PathBuf,
     /// Build directory
-    pub target_directory: PathBuf,
+    pub target_directory: Utf8PathBuf,
+    /// The workspace-level metadata object. Null if non-existent.
+    #[serde(rename = "metadata", default, skip_serializing_if = "is_null")]
+    pub workspace_metadata: serde_json::Value,
+    /// The metadata format version
     version: usize,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
+}
+
+impl Metadata {
+    /// Get the root package of this metadata instance.
+    pub fn root_package(&self) -> Option<&Package> {
+        let root = self.resolve.as_ref()?.root.as_ref()?;
+        self.packages.iter().find(|pkg| &pkg.id == root)
+    }
 }
 
 impl<'a> std::ops::Index<&'a PackageId> for Metadata {
@@ -232,6 +175,9 @@ impl<'a> std::ops::Index<&'a PackageId> for Metadata {
 }
 
 #[derive(Clone, Serialize, Deserialize, Debug)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[non_exhaustive]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
 /// A dependency graph
 pub struct Resolve {
     /// Nodes in a dependencies graph
@@ -239,12 +185,12 @@ pub struct Resolve {
 
     /// The crate for which the metadata was read.
     pub root: Option<PackageId>,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
 }
 
 #[derive(Clone, Serialize, Deserialize, Debug)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[non_exhaustive]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
 /// A node in a dependencies graph
 pub struct Node {
     /// An opaque identifier for a package
@@ -262,12 +208,12 @@ pub struct Node {
     /// Features enabled on the crate
     #[serde(default)]
     pub features: Vec<String>,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
 }
 
 #[derive(Clone, Serialize, Deserialize, Debug)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[non_exhaustive]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
 /// A dependency in a node
 pub struct NodeDep {
     /// The name of the dependency's library target.
@@ -280,12 +226,12 @@ pub struct NodeDep {
     /// This field was added in Rust 1.41.
     #[serde(default)]
     pub dep_kinds: Vec<DepKindInfo>,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
 }
 
 #[derive(Clone, Serialize, Deserialize, Debug)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[non_exhaustive]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
 /// Information about a dependency kind.
 pub struct DepKindInfo {
     /// The kind of dependency.
@@ -301,15 +247,18 @@ pub struct DepKindInfo {
     /// graph. Use Cargo's `--filter-platform` flag if you only want to
     /// include dependencies for a specific platform.
     ///
-    /// [`Display`]: https://doc.rust-lang.org/std/fmt/trait.Display.html
+    /// [`Display`]: std::fmt::Display
     pub target: Option<dependency::Platform>,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
 }
 
-#[derive(Clone, Serialize, Deserialize, Debug)]
-/// A crate
+#[derive(Clone, Serialize, Deserialize, Debug, PartialEq, Eq)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[non_exhaustive]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
+/// One or more crates described by a single `Cargo.toml`
+///
+/// Each [`target`][Package::targets] of a `Package` will be built as a crate.
+/// For more information, see <https://doc.rust-lang.org/book/ch07-01-packages-and-crates.html>.
 pub struct Package {
     /// Name as given in the `Cargo.toml`
     pub name: String,
@@ -331,13 +280,13 @@ pub struct Package {
     pub license: Option<String>,
     /// If the package is using a nonstandard license, this key may be specified instead of
     /// `license`, and must point to a file relative to the manifest.
-    pub license_file: Option<PathBuf>,
+    pub license_file: Option<Utf8PathBuf>,
     /// Targets provided by the crate (lib, bin, example, test, ...)
     pub targets: Vec<Target>,
     /// Features provided by the crate, mapped to the features required by that feature.
     pub features: HashMap<String, Vec<String>>,
     /// Path containing the `Cargo.toml`
-    pub manifest_path: PathBuf,
+    pub manifest_path: Utf8PathBuf,
     /// Categories as given in the `Cargo.toml`
     #[serde(default)]
     pub categories: Vec<String>,
@@ -345,14 +294,22 @@ pub struct Package {
     #[serde(default)]
     pub keywords: Vec<String>,
     /// Readme as given in the `Cargo.toml`
-    pub readme: Option<PathBuf>,
+    pub readme: Option<Utf8PathBuf>,
     /// Repository as given in the `Cargo.toml`
     // can't use `url::Url` because that requires a more recent stable compiler
     pub repository: Option<String>,
+    /// Homepage as given in the `Cargo.toml`
+    ///
+    /// On versions of cargo before 1.49, this will always be [`None`].
+    pub homepage: Option<String>,
+    /// Documentation URL as given in the `Cargo.toml`
+    ///
+    /// On versions of cargo before 1.49, this will always be [`None`].
+    pub documentation: Option<String>,
     /// Default Rust edition for the package
     ///
     /// Beware that individual targets may specify their own edition in
-    /// [`Target::edition`](struct.Target.html#structfield.edition).
+    /// [`Target::edition`].
     #[serde(default = "edition_default")]
     pub edition: String,
     /// Contents of the free form package.metadata section
@@ -360,10 +317,8 @@ pub struct Package {
     /// This contents can be serialized to a struct using serde:
     ///
     /// ```rust
-    /// #[macro_use]
-    /// extern crate serde_json;
-    /// #[macro_use]
-    /// extern crate serde_derive;
+    /// use serde::Deserialize;
+    /// use serde_json::json;
     ///
     /// #[derive(Debug, Deserialize)]
     /// struct SomePackageMetadata {
@@ -380,7 +335,7 @@ pub struct Package {
     /// }
     ///
     /// ```
-    #[serde(default)]
+    #[serde(default, skip_serializing_if = "is_null")]
     pub metadata: serde_json::Value,
     /// The name of a native library the package is linking to.
     pub links: Option<String>,
@@ -390,21 +345,29 @@ pub struct Package {
     ///
     /// This is always `None` if running with a version of Cargo older than 1.39.
     pub publish: Option<Vec<String>>,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
+    /// The default binary to run by `cargo run`.
+    ///
+    /// This is always `None` if running with a version of Cargo older than 1.55.
+    pub default_run: Option<String>,
+    /// The minimum supported Rust version of this package.
+    ///
+    /// This is always `None` if running with a version of Cargo older than 1.58.
+    pub rust_version: Option<VersionReq>,
 }
 
 impl Package {
     /// Full path to the license file if one is present in the manifest
-    pub fn license_file(&self) -> Option<PathBuf> {
-        self.license_file
-            .as_ref()
-            .map(|file| self.manifest_path.join(file))
+    pub fn license_file(&self) -> Option<Utf8PathBuf> {
+        self.license_file.as_ref().map(|file| {
+            self.manifest_path
+                .parent()
+                .unwrap_or(&self.manifest_path)
+                .join(file)
+        })
     }
 
     /// Full path to the readme file if one is present in the manifest
-    pub fn readme(&self) -> Option<PathBuf> {
+    pub fn readme(&self) -> Option<Utf8PathBuf> {
         self.readme
             .as_ref()
             .map(|file| self.manifest_path.join(file))
@@ -412,54 +375,73 @@ impl Package {
 }
 
 /// The source of a package such as crates.io.
-#[derive(Clone, Serialize, Deserialize, Debug)]
+#[derive(Clone, Serialize, Deserialize, Debug, PartialEq, Eq)]
 #[serde(transparent)]
-pub struct Source(String);
+pub struct Source {
+    /// The underlying string representation of a source.
+    pub repr: String,
+}
 
 impl Source {
     /// Returns true if the source is crates.io.
     pub fn is_crates_io(&self) -> bool {
-        self.0 == "registry+https://github.com/rust-lang/crates.io-index"
+        self.repr == "registry+https://github.com/rust-lang/crates.io-index"
     }
 }
 
 impl std::fmt::Display for Source {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        fmt::Display::fmt(&self.0, f)
+        fmt::Display::fmt(&self.repr, f)
     }
 }
 
-#[derive(Clone, Serialize, Deserialize, Debug)]
+#[derive(Clone, Serialize, Deserialize, Debug, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "builder", derive(Builder))]
+#[cfg_attr(feature = "builder", builder(pattern = "owned", setter(into)))]
+#[non_exhaustive]
 /// A single target (lib, bin, example, ...) provided by a crate
 pub struct Target {
     /// Name as given in the `Cargo.toml` or generated from the file name
     pub name: String,
     /// Kind of target ("bin", "example", "test", "bench", "lib")
     pub kind: Vec<String>,
-    /// Almost the same as `kind`, except when an example is a library instad of an executable.
+    /// Almost the same as `kind`, except when an example is a library instead of an executable.
     /// In that case `crate_types` contains things like `rlib` and `dylib` while `kind` is `example`
     #[serde(default)]
+    #[cfg_attr(feature = "builder", builder(default))]
     pub crate_types: Vec<String>,
 
     #[serde(default)]
+    #[cfg_attr(feature = "builder", builder(default))]
     #[serde(rename = "required-features")]
     /// This target is built only if these features are enabled.
     /// It doesn't apply to `lib` targets.
     pub required_features: Vec<String>,
     /// Path to the main source file of the target
-    pub src_path: PathBuf,
+    pub src_path: Utf8PathBuf,
     /// Rust edition for this target
     #[serde(default = "edition_default")]
+    #[cfg_attr(feature = "builder", builder(default = "edition_default()"))]
     pub edition: String,
     /// Whether or not this target has doc tests enabled, and the target is
     /// compatible with doc testing.
     ///
     /// This is always `true` if running with a version of Cargo older than 1.37.
     #[serde(default = "default_true")]
+    #[cfg_attr(feature = "builder", builder(default = "true"))]
     pub doctest: bool,
-    #[doc(hidden)]
-    #[serde(skip)]
-    __do_not_match_exhaustively: (),
+    /// Whether or not this target is tested by default by `cargo test`.
+    ///
+    /// This is always `true` if running with a version of Cargo older than 1.47.
+    #[serde(default = "default_true")]
+    #[cfg_attr(feature = "builder", builder(default = "true"))]
+    pub test: bool,
+    /// Whether or not this target is documented by `cargo doc`.
+    ///
+    /// This is always `true` if running with a version of Cargo older than 1.50.
+    #[serde(default = "default_true")]
+    #[cfg_attr(feature = "builder", builder(default = "true"))]
+    pub doc: bool,
 }
 
 fn default_true() -> bool {
@@ -484,11 +466,24 @@ pub enum CargoOpt {
 /// A builder for configurating `cargo metadata` invocation.
 #[derive(Debug, Clone, Default)]
 pub struct MetadataCommand {
+    /// Path to `cargo` executable.  If not set, this will use the
+    /// the `$CARGO` environment variable, and if that is not set, will
+    /// simply be `cargo`.
     cargo_path: Option<PathBuf>,
+    /// Path to `Cargo.toml`
     manifest_path: Option<PathBuf>,
+    /// Current directory of the `cargo metadata` process.
     current_dir: Option<PathBuf>,
+    /// Output information only about the root package and don't fetch dependencies.
     no_deps: bool,
-    features: Option<CargoOpt>,
+    /// Collections of `CargoOpt::SomeFeatures(..)`
+    features: Vec<String>,
+    /// Latched `CargoOpt::AllFeatures`
+    all_features: bool,
+    /// Latched `CargoOpt::NoDefaultFeatures`
+    no_default_features: bool,
+    /// Arbitrary command line flags to pass to `cargo`.  These will be added
+    /// to the end of the command line invocation.
     other_options: Vec<String>,
 }
 
@@ -501,18 +496,18 @@ impl MetadataCommand {
     /// Path to `cargo` executable.  If not set, this will use the
     /// the `$CARGO` environment variable, and if that is not set, will
     /// simply be `cargo`.
-    pub fn cargo_path(&mut self, path: impl AsRef<Path>) -> &mut MetadataCommand {
-        self.cargo_path = Some(path.as_ref().to_path_buf());
+    pub fn cargo_path(&mut self, path: impl Into<PathBuf>) -> &mut MetadataCommand {
+        self.cargo_path = Some(path.into());
         self
     }
     /// Path to `Cargo.toml`
-    pub fn manifest_path(&mut self, path: impl AsRef<Path>) -> &mut MetadataCommand {
-        self.manifest_path = Some(path.as_ref().to_path_buf());
+    pub fn manifest_path(&mut self, path: impl Into<PathBuf>) -> &mut MetadataCommand {
+        self.manifest_path = Some(path.into());
         self
     }
     /// Current directory of the `cargo metadata` process.
-    pub fn current_dir(&mut self, path: impl AsRef<Path>) -> &mut MetadataCommand {
-        self.current_dir = Some(path.as_ref().to_path_buf());
+    pub fn current_dir(&mut self, path: impl Into<PathBuf>) -> &mut MetadataCommand {
+        self.current_dir = Some(path.into());
         self
     }
     /// Output information only about the root package and don't fetch dependencies.
@@ -521,22 +516,77 @@ impl MetadataCommand {
         self
     }
     /// Which features to include.
+    ///
+    /// Call this multiple times to specify advanced feature configurations:
+    ///
+    /// ```no_run
+    /// # use cargo_metadata::{CargoOpt, MetadataCommand};
+    /// MetadataCommand::new()
+    ///     .features(CargoOpt::NoDefaultFeatures)
+    ///     .features(CargoOpt::SomeFeatures(vec!["feat1".into(), "feat2".into()]))
+    ///     .features(CargoOpt::SomeFeatures(vec!["feat3".into()]))
+    ///     // ...
+    ///     # ;
+    /// ```
+    ///
+    /// # Panics
+    ///
+    /// `cargo metadata` rejects multiple `--no-default-features` flags. Similarly, the `features()`
+    /// method panics when specifying multiple `CargoOpt::NoDefaultFeatures`:
+    ///
+    /// ```should_panic
+    /// # use cargo_metadata::{CargoOpt, MetadataCommand};
+    /// MetadataCommand::new()
+    ///     .features(CargoOpt::NoDefaultFeatures)
+    ///     .features(CargoOpt::NoDefaultFeatures) // <-- panic!
+    ///     // ...
+    ///     # ;
+    /// ```
+    ///
+    /// The method also panics for multiple `CargoOpt::AllFeatures` arguments:
+    ///
+    /// ```should_panic
+    /// # use cargo_metadata::{CargoOpt, MetadataCommand};
+    /// MetadataCommand::new()
+    ///     .features(CargoOpt::AllFeatures)
+    ///     .features(CargoOpt::AllFeatures) // <-- panic!
+    ///     // ...
+    ///     # ;
+    /// ```
     pub fn features(&mut self, features: CargoOpt) -> &mut MetadataCommand {
-        self.features = Some(features);
+        match features {
+            CargoOpt::SomeFeatures(features) => self.features.extend(features),
+            CargoOpt::NoDefaultFeatures => {
+                assert!(
+                    !self.no_default_features,
+                    "Do not supply CargoOpt::NoDefaultFeatures more than once!"
+                );
+                self.no_default_features = true;
+            }
+            CargoOpt::AllFeatures => {
+                assert!(
+                    !self.all_features,
+                    "Do not supply CargoOpt::AllFeatures more than once!"
+                );
+                self.all_features = true;
+            }
+        }
         self
     }
     /// Arbitrary command line flags to pass to `cargo`.  These will be added
     /// to the end of the command line invocation.
-    pub fn other_options(&mut self, options: impl AsRef<[String]>) -> &mut MetadataCommand {
-        self.other_options = options.as_ref().to_vec();
+    pub fn other_options(&mut self, options: impl Into<Vec<String>>) -> &mut MetadataCommand {
+        self.other_options = options.into();
         self
     }
-    /// Runs configured `cargo metadata` and returns parsed `Metadata`.
-    pub fn exec(&mut self) -> Result<Metadata> {
+
+    /// Builds a command for `cargo metadata`.  This is the first
+    /// part of the work of `exec`.
+    pub fn cargo_command(&self) -> Command {
         let cargo = self
             .cargo_path
             .clone()
-            .or_else(|| env::var("CARGO").map(|s| PathBuf::from(s)).ok())
+            .or_else(|| env::var("CARGO").map(PathBuf::from).ok())
             .unwrap_or_else(|| PathBuf::from("cargo"));
         let mut cmd = Command::new(cargo);
         cmd.args(&["metadata", "--format-version", "1"]);
@@ -549,19 +599,34 @@ impl MetadataCommand {
             cmd.current_dir(path);
         }
 
-        if let Some(features) = &self.features {
-            match features {
-                CargoOpt::AllFeatures => cmd.arg("--all-features"),
-                CargoOpt::NoDefaultFeatures => cmd.arg("--no-default-features"),
-                CargoOpt::SomeFeatures(ftrs) => cmd.arg("--features").arg(ftrs.join(",")),
-            };
+        if !self.features.is_empty() {
+            cmd.arg("--features").arg(self.features.join(","));
+        }
+        if self.all_features {
+            cmd.arg("--all-features");
+        }
+        if self.no_default_features {
+            cmd.arg("--no-default-features");
         }
 
         if let Some(manifest_path) = &self.manifest_path {
             cmd.arg("--manifest-path").arg(manifest_path.as_os_str());
         }
         cmd.args(&self.other_options);
-        let output = cmd.output()?;
+
+        cmd
+    }
+
+    /// Parses `cargo metadata` output.  `data` must have been
+    /// produced by a command built with `cargo_command`.
+    pub fn parse<T: AsRef<str>>(data: T) -> Result<Metadata> {
+        let meta = serde_json::from_str(data.as_ref())?;
+        Ok(meta)
+    }
+
+    /// Runs configured `cargo metadata` and returns parsed `Metadata`.
+    pub fn exec(&self) -> Result<Metadata> {
+        let output = self.cargo_command().output()?;
         if !output.status.success() {
             return Err(Error::CargoMetadata {
                 stderr: String::from_utf8(output.stderr)?,
@@ -571,7 +636,6 @@ impl MetadataCommand {
             .lines()
             .find(|line| line.starts_with('{'))
             .ok_or_else(|| Error::NoJson)?;
-        let meta = serde_json::from_str(stdout)?;
-        Ok(meta)
+        Self::parse(stdout)
     }
 }
